/*-----------------------------------------------------------------------
		
	Inline Math library : Bounding box implementation (cpp)
	
	* See imath_box.h for more information

	By Matt Barnett
		(see References in imath.h)
	
------------------------------------------------------------------------*/

#include "imath.h"

/*===========================================
const void ibox3::gldrawSphere()

//very dirty way of display the box
============================================*/

const void ibox3::gldrawSphere()
{
	GLUquadricObj* qobj;
	qobj = gluNewQuadric();

	gluQuadricDrawStyle(qobj,GLU_SILHOUETTE);

	ivector3 areal = pos;
	ivector3 temp;

	temp = bas.x;temp.multiply(center.x);areal.add(temp);
	temp = bas.y;temp.multiply(center.y);areal.add(temp);
	temp = bas.z;temp.multiply(center.z);areal.add(temp);


	glPushMatrix();
		
		glTranslatef(areal.x,areal.y,areal.z);
		gluSphere(qobj, extents.length() ,9,9);

	glPopMatrix();

	gluDeleteQuadric(qobj);
}

/*===========================================
const void ibox3::gldraw()

//very dirty way of display the box
============================================*/
const void ibox3::gldraw()
{
	//pos.gldraw();
	
	//draw the box

	ivector3 h,l;

	h.add(extents);
	l.subtract(extents);

	//could do my own rotation around it
	//from here.

	glPushMatrix();
		
		glTranslatef(pos.x,pos.y,pos.z);

		update();

		//push and add the quaternion matrix
		glPushMatrix();
		
			glMultMatrixf(rm.m);

				glTranslatef(center.x,center.y,center.z);
				
//draw test object here
#ifdef _IBOX3_TESTWITH_TEAPOT
					glEnable(GL_LIGHTING);	
					glRotatef(90,1.0,0.0,0.0);
//draw test object here
					glDisable(GL_LIGHTING);	
#endif	
					glDisable(GL_TEXTURE_2D);	

					//draw the bow
					glBegin(GL_LINE_LOOP);							//top loop
 						glVertex3f(h.v[0],h.v[1],h.v[2]);	
						glVertex3f(l.v[0],h.v[1],h.v[2]);
						glVertex3f(l.v[0],l.v[1],h.v[2]);
						glVertex3f(h.v[0],l.v[1],h.v[2]);
					glEnd();
					glBegin(GL_LINE_LOOP);							//bottom loop
						glVertex3f(h.v[0],h.v[1],l.v[2]);
						glVertex3f(l.v[0],h.v[1],l.v[2]);
						glVertex3f(l.v[0],l.v[1],l.v[2]);
						glVertex3f(h.v[0],l.v[1],l.v[2]);
					glEnd();
					glBegin(GL_LINES);									//the lines inbetween the loops
						glVertex3f(h.v[0],h.v[1],h.v[2]);glVertex3f(h.v[0],h.v[1],l.v[2]);
						glVertex3f(l.v[0],h.v[1],h.v[2]);glVertex3f(l.v[0],h.v[1],l.v[2]);
						glVertex3f(l.v[0],l.v[1],h.v[2]);glVertex3f(l.v[0],l.v[1],l.v[2]);
						glVertex3f(h.v[0],l.v[1],h.v[2]);glVertex3f(h.v[0],l.v[1],l.v[2]);
					glEnd();
					
					glBegin(GL_LINES);
						glColor3f(1.0f,0.0,0.0);glVertex3f(-0.5f,0.0,0.0);glVertex3f(1.2f,0.0,0.0);
						glColor3f(0.0,1.0f,0.0);glVertex3f(0.0,-0.5f,0.0);glVertex3f(0.0,1.2f,0.0);
						glColor3f(0.0,0.0,1.0f);glVertex3f(0.0,0.0,-0.5f);glVertex3f(0.0,0.0,1.2f);
					glEnd();

		glPopMatrix();

		//pop off the quat matrix
		//and draw the basis
	
		bas.gldraw();

		glBegin(GL_LINES);
			glColor3f(1.0f,0.0,0.0);glVertex3f(-0.5f,0.0,0.0);glVertex3f(1.2f,0.0,0.0);
			glColor3f(0.0,1.0f,0.0);glVertex3f(0.0,-0.5f,0.0);glVertex3f(0.0,1.2f,0.0);
			glColor3f(0.0,0.0,1.0f);glVertex3f(0.0,0.0,-0.5f);glVertex3f(0.0,0.0,1.2f);
		glEnd();
	glPopMatrix();	

}

/*===========================================
const bool ibox3::intersect(const ibox3 &b)

	true for bounding box collision detection
	between this and b
	
    (seperating axis test based on
	 gamasutra article)
============================================*/

const bool ibox3::intersect(const ibox3 &b) const
{
	
	//<mat2002>
	
	//mat2002 added offset for center
	//
	//we must pass the real center to the intersect test, adjusted for the pivot/center
	//to do this, we need to transform the position by the center with respect to the basis
	//(this just means moving it from its position to its center along its basis)

	ivector3 areal = pos;
	ivector3 breal = b.pos;

	ivector3 temp;

	temp = bas.x;temp.multiply(center.x);areal.add(temp);
	temp = bas.y;temp.multiply(center.y);areal.add(temp);
	temp = bas.z;temp.multiply(center.z);areal.add(temp);

	temp = b.bas.x;temp.multiply(b.center.x);breal.add(temp);
	temp = b.bas.y;temp.multiply(b.center.y);breal.add(temp);
	temp = b.bas.z;temp.multiply(b.center.z);breal.add(temp);

	//works without centers
	//ivector3 areal = (pos);
	//ivector3 breal = (b.pos);

	ivector3 v =  breal - areal;								//translation, in parent frame
	ivector3 T( bas.x.dot(v),bas.y.dot(v),bas.z.dot(v) );		//translation in A's frame

	float R[3][3];												//rotation matrix
	float ra,rb,t;

	int i,k;

	//calculate rotation matrix R[][]
	R[0][0] = bas.x.dot(b.bas.x);												
	R[0][1] = bas.x.dot(b.bas.y);
	R[0][2] = bas.x.dot(b.bas.z);

	R[1][0] = bas.y.dot(b.bas.x);												
	R[1][1] = bas.y.dot(b.bas.y);
	R[1][2] = bas.y.dot(b.bas.z);

	R[2][0] = bas.z.dot(b.bas.x);												
	R[2][1] = bas.z.dot(b.bas.y);
	R[2][2] = bas.z.dot(b.bas.z);



//	for (i=0;i<3;i++)
//		for (k=0;k<3;k++)
//			printf("R[%i][%i] %f\n",i,k,R[i][k]);


	//a's basis vectors
	for (i=0;i<3;i++)
	{
		ra = extents[i];		
		//ra = extents[i];     //mat2002

		rb = b.extents[0] * fabs(R[i][0]) + b.extents[1] * fabs(R[i][1]) + b.extents[2] * fabs(R[i][2]);
		t = fabs(T[i]);
		if (t > ra + rb)
		{
			#ifdef VERBOSE_SEPERATION
				printf("a basis %i seperates a and b\n",i);
			#endif

			return false;
		}
	}

	//safe to here

	//b's basis vectors 
	for (k=0;k<3;k++)
	{
		ra = extents[0] * fabs(R[0][k]) + extents[1] * fabs(R[1][k]) + extents[2] * fabs(R[2][k]);
		rb = b.extents[k];
		t = fabs(T[0] * R[0][k] + T[1] * R[1][k] + T[2] * R[2][k]);
		if (t > ra + rb) 
		{
			#ifdef VERBOSE_SEPERATION
				printf("b basis %i seperates a and b\n",k);
			#endif

			return false;
		}
	}
	
	//9 cross products
	
	//all the a0s
	
	ra = extents[1] * fabs(R[2][0]) + extents[2] * fabs(R[1][0]);
	rb = b.extents[1] * fabs(R[0][2]) + b.extents[2] * fabs(R[0][1]);
	t = fabs(T[2] * R[1][0] - T[1] * R[2][0]);
	if (t > ra + rb) 
	{
		#ifdef VERBOSE_SEPERATION
			printf("a0*b0 seperates a and b\n",k);
		#endif
		
		return false;
	}
	ra = extents[1] * fabs(R[2][1]) + extents[2] * fabs(R[1][1]);
	rb = b.extents[0] * fabs(R[0][2]) + b.extents[2] * fabs(R[0][0]);
	t = fabs(T[2] * R[1][1] - T[1] * R[2][1]);
	if (t > ra + rb)
	{

		#ifdef VERBOSE_SEPERATION
			printf("a0*b1 seperates a and b\n",k);
		#endif

		return false;
	}

	ra = extents[1] * fabs(R[2][2]) + extents[2] * fabs(R[1][2]);
	rb = b.extents[0] * fabs(R[0][1]) + b.extents[1] * fabs(R[0][0]);
	t = fabs(T[2] * R[1][2] - T[1] * R[2][2]);
	if (t > ra + rb) 
	{

		#ifdef VERBOSE_SEPERATION
			printf("a0*b2 seperates a and b\n",k);
		#endif

		return false;
	}


	//all the a1s

	ra = extents[0] * fabs(R[2][0]) + extents[2] * fabs(R[0][0]);
	rb = b.extents[1] * fabs(R[1][2]) + b.extents[2] * fabs(R[1][1]);
	t = fabs(T[0] * R[2][0] - T[2] * R[0][0]);
	if (t > ra + rb) 
	{

		#ifdef VERBOSE_SEPERATION
			printf("a1*b0 seperates a and b\n",k);
		#endif

		return false;
	}

	ra = extents[0] * fabs(R[2][1]) + extents[2] * fabs(R[0][1]);
	rb = b.extents[0] * fabs(R[1][2]) + b.extents[2] * fabs(R[1][0]);
	t = fabs(T[0] * R[2][1] - T[2] * R[0][1]);
	if (t > ra + rb) 
	{
		#ifdef VERBOSE_SEPERATION
			printf("a1*b1 seperates a and b\n",k);
		#endif
			
		return false;
	}

	ra = extents[0] * fabs(R[2][2]) + extents[2] * fabs(R[0][2]);
	rb = b.extents[0] * fabs(R[1][1]) + b.extents[1] * fabs(R[1][0]);
	t = fabs(T[0] * R[2][2] - T[2] * R[0][2]);
	if (t > ra + rb) 
	{
		#ifdef VERBOSE_SEPERATION
			printf("a1*b2 seperates a and b\n",k);
		#endif
			
		return false;
	}


	//all the a2s


	ra = extents[0] * fabs(R[1][0]) + extents[1] * fabs(R[0][0]);
	rb = b.extents[1] * fabs(R[2][2]) + b.extents[2] * fabs(R[2][1]);
	t = fabs(T[1] * R[0][0] - T[0] * R[1][0]);
	if (t > ra + rb) 
	{
		#ifdef VERBOSE_SEPERATION
			printf("a2*b0 seperates a and b\n",k);	
		#endif
			
		return false;
	}

	ra = extents[0] * fabs(R[1][1]) + extents[1] * fabs(R[0][1]);
	rb = b.extents[0] * fabs(R[2][2]) + b.extents[2] * fabs(R[2][0]);
	t = fabs(T[1] * R[0][1] - T[0] * R[1][1]);
	if (t > ra + rb)
	{
		#ifdef VERBOSE_SEPERATION
			printf("a2*b1 seperates a and b\n",k);	
		#endif
			
		return false;
	}

	ra = extents[0] * fabs(R[1][2]) + extents[1] * fabs(R[0][2]);
	rb = b.extents[0] * fabs(R[2][1]) + b.extents[1] * fabs(R[2][0]);
	t = fabs(T[1] * R[0][2] - T[0] * R[1][2]);
	if (t > ra + rb) 
	{
		#ifdef VERBOSE_SEPERATION
			printf("a2*b2 seperates a and b\n",k);
		#endif
			
		return false;
	}

	//disturbed laughter
	return true;
}