/*---------------------------------------------
		
	MatNN library 

	By Matt Barnett
		(see References)

	-----------
	description
	-----------

		This file just keeps some ugly private NeuralNet
		classes out of the main header

	------------
	classes
	------------

		Neuron
		NeuronLayer	

	------------
	class design
	------------

	-----------
	References:	
	-----------

		* See matnn.h.
	
---------------------------------------------*/


//-------- headers etc -------
//---------------------------- 

#ifndef MATNN_NEURON_H_HEADER_132424623254
#define MATNN_NEURON_H_HEADER_132424623254

//----------------------------
//----------------------------

namespace MatLayeredNeuralNetLib
{
	//private class for neurons (clients cant mess with neurons)
	class Neuron
	{
		friend class NeuronLayer;

		private:

			Neuron(Neuron &copy);				//copy
			Neuron &operator=(Neuron &rhs);		//assign

			Neuron(int in_inputs,float &in_sig_response);
			~Neuron();

			inline float Calc(float *inputs);
			void RandomizeWeights();
			inline void Mutate(float chance,float extent);

			float *weights;						//array of weights, owned by this neuron
			int num_inputs;
			float &sig_response;
	};	

	//private class for neuron layers (clients cant mess with layers)
	class NeuronLayer
	{
		friend class NeuralNet;
		friend class TrainableNeuralNet;

		private:

			NeuronLayer(NeuronLayer &copy);				//copy
			NeuronLayer &operator=(NeuronLayer &rhs);	//assign

			NeuronLayer(int in_neurons,int in_num_inputs,float *finputs);
			~NeuronLayer();

			inline void RunLayer();
			void Randomize();
			inline int Mutate(float chance,float extent);

			inline float GetOutput(int neuron_index)			 { return output[neuron_index]; }
			inline void  SetResponse(float in_sig_response)		 { sig_response = in_sig_response; }

			int num_neurons;
			float *input;					//float array pointer (points to parent layers outputs);
			float *output;					//float array pointer (owned by this layer)
			Neuron **neurons;				//array of pointers to neurons
			float sig_response;
	};	
}

#endif